==> ember-1.7.tar.gz: ember-1.7/src/libember/mod_str.c <==
/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: mod_str.c,v 1.13 2002/09/23 03:51:59 bambi Exp $
**
*/

/*
** Module	: String Handling  Library Module
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/file.h>
#include <errno.h>

#include <config/config.h>

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#include "ember.h"
#include <regexp/regexp.h>



/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/


static char *_trSpec(script, spec)
	ember	*script;
	char	*spec;
{
	char	res[4096];
	register char	*cp1,
			*cp2,
			c;
			

	bzero(res,4096);
	cp1 = spec;
	cp2 = res;
	while(*cp1)
	{
		if (*cp1 == '-')
		{
			if (cp1 == spec)
			{
				eRunSetError(script,EMBER_TYPE_TEXT,"Bad TR Format");
				return(NULL);
			}
			if (*(cp1+1) < (*cp1-1))
			{
				eRunSetError(script,EMBER_TYPE_TEXT,"Bad TR Format");
				return(NULL);
			}
			c = *(cp1-1);
			while(c < *(cp1+1))
			{
				c++;
				*cp2++ = c;
			}
			cp1+=2;
			continue;
		}
		*cp2++ = *cp1++;
	}
	return((char *)strdup(res));
}




/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/



int doChop(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str;

	str = emTextArg(args);
	str[strlen(str) - 1]=0;
       	eSymSetTextValue(script,&script->returnValue, str);
	return(0);
}


int doStrSeg(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str,
		*new;
	int	start,
		end,
		length;

	str = emTextArg(args);
	start = emIntArg(args->next);
	end = emIntArg(args->next->next);
	if ( (start < 0) || (start > end) )
	{
		eRunSetError(script,EMBER_TYPE_TEXT, "Bad string index");
		return(0);
	}
	length = end - start + 1;
	new = (char *)malloc(length+1);
	(void)bzero(new, length+1);
	strncpy(new, str + start, length);
       	eSymSetTextValue(script,&script->returnValue, new);
	free(new);
	return(0);
}


int doSub(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*spec1,
		*spec2,
		*str,
		*buf,
		*oldBuf;
	register char	*cpb,
			*cp;
	int	len,
		curLen,
		spec1Len,
		spec2Len;

	str = emTextArg(args);
	spec1 = emTextArg(args->next);
	spec2 = emTextArg(args->next->next);
	curLen = strlen(str);
	spec1Len = strlen(spec1);
	spec2Len = strlen(spec2);
	len = 0;

	buf = (char *)malloc(curLen+1);
	(void)bzero(buf,curLen);
	cpb = buf;
	cp = str;
	while (*cp)
	{
		if (*cp != *spec1)
		{
			*cpb++ = *cp++;	
			len++;
			continue;
		}
		if (strncmp(cp,spec1,spec1Len) != 0)
		{
			*cpb++ = *cp++;	
			len++;
			continue;
		}
		len += spec2Len ;
		if (spec2Len > spec1Len)
		{
			oldBuf = buf;
			buf = (char *)realloc(buf, curLen + 
				(spec2Len-spec1Len)+1);
			cpb = buf + (cpb - oldBuf);
			curLen += (spec2Len-spec1Len)+1;
		}
		strcpy(cpb,spec2);
		cpb += spec2Len;
		cp += spec1Len;
	}
	*cpb = 0;
       	eSymSetTextValue(script,&script->returnValue, buf);
	free(buf);
	return(0);
}


int doTr(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*trans1,
		*trans2,
		*spec1,
		*spec2,
		*str;
	register char	*cps,
			*cp;

	str = emTextArg(args);
	spec1 = emTextArg(args->next);
	spec2 = emTextArg(args->next->next);

	if((trans1 = _trSpec(script,spec1)) == NULL)
		return(0);
	if((trans2 = _trSpec(script,spec2)) == NULL)
	{
		free(trans1);
		return(0);
	}
	cps = str;
	while(*cps)
	{
		cp = trans1;
		while(*cp)
		{
			if (*cp == *cps)
			{
				*cps = *(char *)(trans2 + (cp-trans1));
				break;
			}
			cp++;
		}
		cps++;
	}
       	eSymSetTextValue(script,&script->returnValue, str);
	free(trans1);
	free(trans2);
	return(0);
}



int doSubStr(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*reg,
		*source,
		*sub,
		buf[4096];
	regexp	*prog;

	source = emTextArg(args);
	reg = emTextArg(args->next);
	sub = emTextArg(args->next->next);

	prog = regcomp(reg);
	regexec(prog,source);
	(void)bzero(buf,sizeof(buf));
	regsub(prog,sub,buf);
	free(prog);
       	eSymSetTextValue(script,&script->returnValue, buf);
	return(0);
}


int doSplit(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*data,
		sep,
		*cp,
		*cp1;
	int	index = 0;
	e_value	val;

	data = emTextArg(args);
	sep = *(emTextArg(args->next));

	cp1 = cp = data;
	while(*cp)
	{
		if (*cp == sep)
		{
			*cp++ = 0;
			eSymSetTextValue(script, &val, cp1);
			eSymSetArrayElement(script, &script->returnValue,
				&val, index);
			eSymFreeValue(script, &val);
			cp1=cp;
			index++;
		}
		else
			cp++;
	}
	eSymSetTextValue(script, &val, cp1);
	eSymSetArrayElement(script, &script->returnValue, &val, index);
	eSymFreeValue(script, &val);
	return(0);
}



int doMerge(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*tok;
	int	num,
		count;
	char	buf[2048];
	e_value	*curVal;

	tok = emTextArg(args->next);
	num = eSymGetNumArrayElements(script, emArrayArg(args));
	count = 0;
	*buf = 0;
	while(count < num)
	{
		curVal = eSymGetArrayElement(script,emArrayArg(args),count);
		if (count > 0)
			strcat(buf,tok);
		if (curVal->value.textVal)
			strcat(buf, curVal->value.textVal);
		count++;
	}
       	eSymSetTextValue(script,&script->returnValue, buf);
	return(0);
}


int doTokSplit(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*data,
		sep,
		*cp,
		*cp1;
	int	index = 0;
	e_value	val;

	data = emTextArg(args);
	sep = *(emTextArg(args->next));

	cp1 = cp = data;
	while(*cp)
	{
		if (*cp == sep)
		{
			*cp++ = 0;
			eSymSetTextValue(script, &val, cp1);
			eSymSetArrayElement(script, &script->returnValue,
				&val, index);
			eSymFreeValue(script, &val);
			while(*cp == sep)
				cp++;
			cp1=cp;
			index++;
		}
		else
			cp++;
	}
	eSymSetTextValue(script, &val, cp1);
	eSymSetArrayElement(script, &script->returnValue, &val, index);
	eSymFreeValue(script, &val);
	return(0);
}



int doExpandText(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*old,
		*new;

	old = emTextArg(args);
	new = eRunExpandText(script,old);
	if (!new)
	{
		eRunSetError(script,EMBER_TYPE_TEXT, script->errorMsg);
		return(0);
	}
       	eSymSetTextValue(script,&script->returnValue, new);
	return(0);
}

int doFatal(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*msg;

	msg = emTextArg(args);
	strcpy(script->errorMsg, msg);
	return(-1);
}



int doCrypt(script, args)
        ember	*script;
	e_stack	*args;
{
        char    *res,
		*salt,
                *pw;
	char *fcrypt();

        pw = emTextArg(args);
        salt = emTextArg(args->next);
        res = (char *)fcrypt(pw,salt);
       	eSymSetTextValue(script,&script->returnValue, res);
	return(0);
}



int doIncludeFile(script, args)
        ember	*script;
	e_stack	*args;
{
        char    *filename,
                buf[4096];
        int     fd,
                numBytes;

	if (script->www.parseHtml && script->www.headersSent == 0)
	{
		eWwwSendResponse(script, NULL);
		eWwwSendHeaders(script);
	}
	fflush(stdout);
	filename = emTextArg(args);
	fd = open(filename,O_RDONLY,0);
	if (fd < 0)
	{
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
		return(0);
	}
	while ((numBytes = read(fd,buf,4096)) > 0)
	{
		write(fileno(stdout),buf,numBytes);
	}
	close(fd);
       	eSymSetIntValue(script,&script->returnValue, 0);
	return(0);
}



int doCharToAscii(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str;
	int	ascii;

	str = emTextArg(args);
	ascii = (int)(char)*str;
       	eSymSetIntValue(script,&script->returnValue, ascii);
	return(0);
}



int doAsciiToChar(script, args)
	ember	*script;
	e_stack	*args;
{
	char	str[2];
	int	ascii;

	ascii = emIntArg(args);
	str[0] = (char)ascii;
	str[1] = 0;
       	eSymSetTextValue(script,&script->returnValue, str);
	return(0);
}



int doSetChar(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*new,
		*old,
		*val;
	int	offset;

	old = emTextArg(args);
	val = emTextArg(args->next);
	offset = emIntArg(args->next->next);
	if (offset > strlen(old))
	{
		eRunSetError(script,EMBER_TYPE_TEXT, 
			"Attempt to insert beyond the strings boundary");
		return(0);
	}
	new = malloc(strlen(old) + 1);
	strcpy(new,old);
	new[offset] = *val;
	new[strlen(new)] = 0;
       	eSymSetTextValue(script,&script->returnValue, new);
	free(new);
	return(0);
}


int doGetChar(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str,
		val[2];
	int	offset;

	str = emTextArg(args);
	offset = emIntArg(args->next);
	if (offset > strlen(str))
	{
		eRunSetError(script,EMBER_TYPE_TEXT, 
			"Attempt to access beyond the strings boundary");
		return(0);
	}
	val[0] = str[offset];
	val[1] = 0;
       	eSymSetTextValue(script,&script->returnValue, val);
	return(0);
}

==> ember-1.8.tar.gz: ember-1.8/src/libember/mod_str.c <==
/*
** Copyright (c) 2001  Hughes Technologies Pty Ltd.  All rights
** reserved.  Terms under which this software may be used or copied are
** provided in the  specific license associated with this product.
**
** Hughes Technologies disclaims all warranties with regard to this 
** software, including all implied warranties of merchantability and 
** fitness, in no event shall Hughes Technologies be liable for any 
** special, indirect or consequential damages or any damages whatsoever 
** resulting from loss of use, data or profits, whether in an action of 
** contract, negligence or other tortious action, arising out of or in 
** connection with the use or performance of this software.
**
**
** $Id: mod_str.c,v 1.16 2006/05/20 07:01:17 bambi Exp $
**
*/

/*
** Module	: String Handling  Library Module
** Purpose	: 
** Exports	: 
** Depends Upon	: 
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/file.h>
#include <errno.h>

#include <config/config.h>

#ifdef HAVE_STRINGS_H
#  include <strings.h>
#endif

#include "ember.h"
#include <regexp/regexp.h>



/**************************************************************************
** GLOBAL VARIABLES
**************************************************************************/


/**************************************************************************
** PRIVATE ROUTINES
**************************************************************************/


static char *_trSpec(script, spec)
	ember	*script;
	char	*spec;
{
	char	res[4096];
	register char	*cp1,
			*cp2,
			c;
			

	bzero(res,4096);
	cp1 = spec;
	cp2 = res;
	while(*cp1)
	{
		if (*cp1 == '-')
		{
			if (cp1 == spec)
			{
				eRunSetError(script,EMBER_TYPE_TEXT,"Bad TR Format");
				return(NULL);
			}
			if (*(cp1+1) < (*cp1-1))
			{
				eRunSetError(script,EMBER_TYPE_TEXT,"Bad TR Format");
				return(NULL);
			}
			c = *(cp1-1);
			while(c < *(cp1+1))
			{
				c++;
				*cp2++ = c;
			}
			cp1+=2;
			continue;
		}
		*cp2++ = *cp1++;
	}
	return((char *)strdup(res));
}




/**************************************************************************
** PUBLIC ROUTINES
**************************************************************************/



int doChop(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str;

	str = emTextArg(args);
	str[strlen(str) - 1]=0;
       	eSymSetTextValue(script,&script->returnValue, str);
	return(0);
}


int doStrSeg(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str,
		*new;
	int	start,
		end,
		length;

	str = emTextArg(args);
	start = emIntArg(args->next);
	end = emIntArg(args->next->next);
	if ( (start < 0) || (start > end) )
	{
		eRunSetError(script,EMBER_TYPE_TEXT, "Bad string index");
		return(0);
	}
	length = end - start + 1;
	new = (char *)malloc(length+1);
	(void)bzero(new, length+1);
	strncpy(new, str + start, length);
       	eSymSetTextValue(script,&script->returnValue, new);
	free(new);
	return(0);
}


int doSub(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*spec1,
		*spec2,
		*str,
		*buf,
		*oldBuf;
	register char	*cpb,
			*cp;
	int	len,
		curLen,
		spec1Len,
		spec2Len;

	str = emTextArg(args);
	spec1 = emTextArg(args->next);
	spec2 = emTextArg(args->next->next);
	curLen = strlen(str);
	spec1Len = strlen(spec1);
	spec2Len = strlen(spec2);
	len = 0;

	buf = (char *)malloc(curLen+1);
	(void)bzero(buf,curLen);
	cpb = buf;
	cp = str;
	while (*cp)
	{
		if (*cp != *spec1)
		{
			*cpb++ = *cp++;	
			len++;
			continue;
		}
		if (strncmp(cp,spec1,spec1Len) != 0)
		{
			*cpb++ = *cp++;	
			len++;
			continue;
		}
		len += spec2Len ;
		if (spec2Len > spec1Len)
		{
			oldBuf = buf;
			buf = (char *)realloc(buf, curLen + 
				(spec2Len-spec1Len)+1);
			cpb = buf + (cpb - oldBuf);
			curLen += (spec2Len-spec1Len)+1;
		}
		strcpy(cpb,spec2);
		cpb += spec2Len;
		cp += spec1Len;
	}
	*cpb = 0;
       	eSymSetTextValue(script,&script->returnValue, buf);
	free(buf);
	return(0);
}


int doTr(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*trans1,
		*trans2,
		*spec1,
		*spec2,
		*str;
	register char	*cps,
			*cp;

	str = emTextArg(args);
	spec1 = emTextArg(args->next);
	spec2 = emTextArg(args->next->next);

	if((trans1 = _trSpec(script,spec1)) == NULL)
		return(0);
	if((trans2 = _trSpec(script,spec2)) == NULL)
	{
		free(trans1);
		return(0);
	}
	cps = str;
	while(*cps)
	{
		cp = trans1;
		while(*cp)
		{
			if (*cp == *cps)
			{
				*cps = *(char *)(trans2 + (cp-trans1));
				break;
			}
			cp++;
		}
		cps++;
	}
       	eSymSetTextValue(script,&script->returnValue, str);
	free(trans1);
	free(trans2);
	return(0);
}



int doSubStr(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*reg,
		*source,
		*sub,
		buf[4096];
	regexp	*prog;

	source = emTextArg(args);
	reg = emTextArg(args->next);
	sub = emTextArg(args->next->next);

	prog = local_regcomp(reg);
	local_regexec(prog,source);
	(void)bzero(buf,sizeof(buf));
	local_regsub(prog,sub,buf);
	free(prog);
       	eSymSetTextValue(script,&script->returnValue, buf);
	return(0);
}


int doSplit(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*data,
		sep,
		*cp,
		*cp1;
	int	index = 0;
	e_value	val;

	data = emTextArg(args);
	sep = *(emTextArg(args->next));

	cp1 = cp = data;
	while(*cp)
	{
		if (*cp == sep)
		{
			*cp++ = 0;
			eSymSetTextValue(script, &val, cp1);
			eSymSetArrayElement(script, &script->returnValue,
				&val, index);
			eSymFreeValue(script, &val);
			cp1=cp;
			index++;
		}
		else
			cp++;
	}
	eSymSetTextValue(script, &val, cp1);
	eSymSetArrayElement(script, &script->returnValue, &val, index);
	eSymFreeValue(script, &val);
	return(0);
}



int doMerge(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*tok;
	int	num,
		count;
	char	buf[2048];
	e_value	*curVal;

	tok = emTextArg(args->next);
	num = eSymGetNumArrayElements(script, emArrayArg(args));
	count = 0;
	*buf = 0;
	while(count < num)
	{
		curVal = eSymGetArrayElement(script,emArrayArg(args),count);
		if (count > 0)
			strcat(buf,tok);
		if (curVal->value.textVal)
			strcat(buf, curVal->value.textVal);
		count++;
	}
       	eSymSetTextValue(script,&script->returnValue, buf);
	return(0);
}


int doTokSplit(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*data,
		sep,
		*cp,
		*cp1;
	int	index = 0;
	e_value	val;

	data = emTextArg(args);
	sep = *(emTextArg(args->next));

	cp1 = cp = data;
	while(*cp)
	{
		if (*cp == sep)
		{
			*cp++ = 0;
			eSymSetTextValue(script, &val, cp1);
			eSymSetArrayElement(script, &script->returnValue,
				&val, index);
			eSymFreeValue(script, &val);
			while(*cp == sep)
				cp++;
			cp1=cp;
			index++;
		}
		else
			cp++;
	}
	eSymSetTextValue(script, &val, cp1);
	eSymSetArrayElement(script, &script->returnValue, &val, index);
	eSymFreeValue(script, &val);
	return(0);
}



int doExpandText(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*old,
		*new;

	old = emTextArg(args);
	new = eRunExpandText(script,old);
	if (!new)
	{
		eRunSetError(script,EMBER_TYPE_TEXT, script->errorMsg);
		return(0);
	}
       	eSymSetTextValue(script,&script->returnValue, new);
	return(0);
}

int doFatal(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*msg;

	msg = emTextArg(args);
	strcpy(script->errorMsg, msg);
	return(-1);
}



int doCrypt(script, args)
        ember	*script;
	e_stack	*args;
{
        char    *res,
		*salt,
                *pw;
	char *fcrypt();

        pw = emTextArg(args);
        salt = emTextArg(args->next);
        res = (char *)fcrypt(pw,salt);
       	eSymSetTextValue(script,&script->returnValue, res);
	return(0);
}



int doIncludeFile(script, args)
        ember	*script;
	e_stack	*args;
{
        char    *filename,
                buf[4096];
        int     fd,
                numBytes;

#ifdef EMBER_API_VER2
	if (script->www.outputHtml && script->www.headersSent == 0)
#else
	if (script->www.headersSent == 0)
#endif
	{
		eWwwSendResponse(script, NULL);
		eWwwSendHeaders(script);
	}
	fflush(stdout);
	filename = emTextArg(args);
	fd = open(filename,O_RDONLY,0);
	if (fd < 0)
	{
		eRunSetError(script,EMBER_TYPE_INT, strerror(errno));
		return(0);
	}
	while ((numBytes = read(fd,buf,4096)) > 0)
	{
		write(fileno(stdout),buf,numBytes);
	}
	close(fd);
       	eSymSetIntValue(script,&script->returnValue, 0);
	return(0);
}



int doCharToAscii(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str;
	int	ascii;

	str = emTextArg(args);
	ascii = (int)(char)*str;
       	eSymSetIntValue(script,&script->returnValue, ascii);
	return(0);
}



int doAsciiToChar(script, args)
	ember	*script;
	e_stack	*args;
{
	char	str[2];
	int	ascii;

	ascii = emIntArg(args);
	str[0] = (char)ascii;
	str[1] = 0;
       	eSymSetTextValue(script,&script->returnValue, str);
	return(0);
}



int doSetChar(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*new,
		*old,
		*val;
	int	offset;

	old = emTextArg(args);
	val = emTextArg(args->next);
	offset = emIntArg(args->next->next);
	if (offset > strlen(old))
	{
		eRunSetError(script,EMBER_TYPE_TEXT, 
			"Attempt to insert beyond the strings boundary");
		return(0);
	}
	new = malloc(strlen(old) + 1);
	strcpy(new,old);
	new[offset] = *val;
	new[strlen(new)] = 0;
       	eSymSetTextValue(script,&script->returnValue, new);
	free(new);
	return(0);
}


int doGetChar(script, args)
	ember	*script;
	e_stack	*args;
{
	char	*str,
		val[2];
	int	offset;

	str = emTextArg(args);
	offset = emIntArg(args->next);
	if (offset > strlen(str))
	{
		eRunSetError(script,EMBER_TYPE_TEXT, 
			"Attempt to access beyond the strings boundary");
		return(0);
	}
	val[0] = str[offset];
	val[1] = 0;
       	eSymSetTextValue(script,&script->returnValue, val);
	return(0);
}
